# -*- coding: utf-8 -*-
# -*- Python Version: 3.7 -*-

"""Functions to set baseline PHPP windows."""

from PHX.PHPP.phpp_app import PHPPConnection

from ph_baseliner.codes.model import BaselineCode
from ph_baseliner.codes.lighting_space_types import space_type_map

def find_lighting_installed_power(_baseline_code: BaselineCode, _program_name: str) -> float:
    """Find the baseline lighting installed power density for a given PHPP space."""

    # -- First, find the building-code name for the PHPP space type
    code_name = space_type_map.get(_program_name, None)

    if not code_name:
        msg = f"Error: The PHPP Space-Type name {_program_name} does not have a "\
                "corresponding building-code name?"
        raise Exception(msg)

    # -- Then, find the lighting installed power density for that building-code name
    return _baseline_code.tables.lighting_area_method.LPD[code_name]

def set_baseline_lighting_installed_power_density(_phpp_conn: PHPPConnection, _baseline_code: BaselineCode):
    """Set the PHPP 'Electricity non-res' worksheet baseline lighting installed power density.

    Arguments:
    ----------
        _phpp_conn: phpp_app.PHPPConnection
            The PHPPConnection object
        _baseline_code: BaselineCode
            The BaselineCode object
    """
    
    for row_num in _phpp_conn.elec_non_res.lighting.used_lighting_row_numbers:
        row_data = _phpp_conn.elec_non_res.lighting.get_lighting_row_data(row_num)
        code_LPD = find_lighting_installed_power(_baseline_code, row_data.utilization_profile_name)
        _phpp_conn.elec_non_res.lighting.set_lighting_power_density(row_num, code_LPD)


# -*- coding: utf-8 -*-
# -*- Python Version: 3.7 -*-

"""Functions for setting the PHPP envelope constructions to the baseline values. """

from PHX.PHPP import phpp_app
from PHX.PHPP.phpp_model.areas_surface import ExistingSurfaceRow
from PHX.model.enums.building import ComponentFaceType, ComponentExposureExterior

from ph_baseliner.codes.model import BaselineCode
from ph_baseliner.codes.options import ClimateZones, Use_Groups
from ph_baseliner.phpp.u_values import (
    create_baseline_constructions,
    add_baseline_constructions_to_phpp,
    BaselineConstructionPHPPids,
)


def set_baseline_envelope_constructions(
    phpp_conn: phpp_app.PHPPConnection,
    baseline_code: BaselineCode,
    climate_zone: ClimateZones,
    use_group: Use_Groups,
) -> None:
    """Set the PHPP Areas worksheet envelope constructions to the baseline values.

    Will create and add new Constructions to the PHPP U-Values worksheet, and the
    PHPP Areas worksheet surfaces will each be updated to use the new constructions.

    Arguments:
    ----------
        * phpp_conn: phpp_app.PHPPConnection:
            The PHPP Connection
        * baseline_code: BaselineCode:
            The baseline code
        * climate_zone: ClimateZones:
            The climate zone
        * use_group: Use_Groups:
            The use group
    """
    baseline_constructions = create_baseline_constructions(
        baseline_code, climate_zone, use_group
    )
    construction_phpp_ids = add_baseline_constructions_to_phpp(
        phpp_conn, baseline_constructions
    )
    reset_phpp_areas_constructions(phpp_conn, construction_phpp_ids)


def reset_phpp_areas_constructions(
    phpp_conn: phpp_app.PHPPConnection, construction_ids: BaselineConstructionPHPPids
) -> None:
    """Reset the PHPP Areas worksheet to the baseline constructions."""

    def row_is_exposed_wall(row_data: ExistingSurfaceRow) -> bool:
        """Exposed wall is a wall that is not a ground floor."""
        return (
            row_data.face_type == ComponentFaceType.WALL
            and row_data.face_exposure == ComponentExposureExterior.EXTERIOR
        )

    def row_is_ground_wall(row_data: ExistingSurfaceRow) -> bool:
        """Ground wall is a wall that is a ground floor."""
        return (
            row_data.face_type == ComponentFaceType.WALL
            and row_data.face_exposure == ComponentExposureExterior.GROUND
        )

    def row_is_roof(row_data: ExistingSurfaceRow) -> bool:
        """Roof is a roof that is not a ground floor."""
        return (
            row_data.face_type == ComponentFaceType.ROOF_CEILING
            and row_data.face_exposure == ComponentExposureExterior.EXTERIOR
        )

    def row_is_ground_floor(row_data: ExistingSurfaceRow) -> bool:
        """Ground floor is a floor that is a ground floor."""
        return (
            row_data.face_type == ComponentFaceType.FLOOR
            and row_data.face_exposure == ComponentExposureExterior.GROUND
        )

    def row_is_exposed_floor(row_data: ExistingSurfaceRow) -> bool:
        """Exposed floor is a floor that is not a ground floor."""
        return (
            row_data.face_type == ComponentFaceType.FLOOR
            and row_data.face_exposure == ComponentExposureExterior.EXTERIOR
        )

    for i, surface_row_data in phpp_conn.areas.surfaces.all_surface_rows:
        if surface_row_data.no_name:
            continue

        if row_is_roof(surface_row_data):
            phpp_conn.areas.set_surface_row_construction(i, construction_ids.roof)
            phpp_conn.areas.set_surface_row_solar_absorptivity(i, 0.75)
            phpp_conn.areas.set_surface_row_emissivity(i, 0.90)
        elif row_is_exposed_wall(surface_row_data):
            phpp_conn.areas.set_surface_row_construction(
                i, construction_ids.exposed_wall
            )
            phpp_conn.areas.set_surface_row_solar_absorptivity(i, 0.75)
            phpp_conn.areas.set_surface_row_emissivity(i, 0.90)
        elif row_is_ground_wall(surface_row_data):
            phpp_conn.areas.set_surface_row_construction(
                i, construction_ids.ground_wall
            )
        elif row_is_exposed_floor(surface_row_data):
            phpp_conn.areas.set_surface_row_construction(
                i, construction_ids.exposed_floor
            )
            phpp_conn.areas.set_surface_row_solar_absorptivity(i, 0.75)
            phpp_conn.areas.set_surface_row_emissivity(i, 0.90)
        elif row_is_ground_floor(surface_row_data):
            phpp_conn.areas.set_surface_row_construction(
                i, construction_ids.ground_floor
            )



# -*- coding: utf-8 -*-
# -*- Python Version: 3.7 -*-

"""Functions to set baseline PHPP windows."""

from PHX.PHPP import phpp_app

from ph_baseliner.codes.model import BaselineCode
from ph_baseliner.codes.options import ClimateZones, PF_Groups, Use_Groups
from ph_baseliner.phpp.components import (
    add_new_baseline_window_glazing,
    add_new_baseline_window_frame,
    create_new_baseline_window_glazing,
)


def get_baseline_u_value(
    _baseline_code: BaselineCode, _climate_zone: ClimateZones, _use_group: Use_Groups
) -> float:
    """Get the baseline U-Value for the specified climate zone."""
    all_u_values = _baseline_code.get_window_u_values()
    cz_uvalues = all_u_values.get_u_value_for_climate(_climate_zone)
    u_value = cz_uvalues.get_u_values_for_use_group(_use_group)
    return u_value


def get_baseline_SHGC(
    _baseline_code: BaselineCode, _climate_zone: ClimateZones, _pf_group: PF_Groups
) -> float:
    """Get the baseline SHGC for the specified climate zone and Projection Factor group."""
    all_shgcs = _baseline_code.get_window_shgcs()
    cz_shgcs = all_shgcs.get_shgcs_for_climate(_climate_zone)
    pf_shgc = cz_shgcs.get_shgc_for_pf(_pf_group)
    return pf_shgc


def set_baseline_window_construction(
    _phpp_conn: phpp_app.PHPPConnection,
    _baseline_code: BaselineCode,
    _climate_zone: ClimateZones,
    _pf_group: PF_Groups,
    _use_group: Use_Groups,
) -> None:
    """Set the baseline window construction in the PHPP Windows Worksheet.

    Arguments:
    ----------
        phpp_conn: phpp_app.PHPPConnection
            The PHPPConnection object
        baseline_code: BaselineCode
            The BaselineCode object
    """

    # -- Get the baseline values for U-Value and SHGC
    u_value = get_baseline_u_value(_baseline_code, _climate_zone, _use_group)
    shgc = get_baseline_SHGC(_baseline_code, _climate_zone, _pf_group)

    # -- Create the new baseline window construction
    baseline_phx_window = create_new_baseline_window_glazing(u_value, shgc)
    phpp_glazing_id = add_new_baseline_window_glazing(_phpp_conn, baseline_phx_window)
    phpp_frame_id = add_new_baseline_window_frame(_phpp_conn, baseline_phx_window)

    # -- Set the window constructions in the Windows Worksheet
    for row_num in _phpp_conn.windows.used_window_row_numbers:
        _phpp_conn.windows.set_single_window_construction_ids(
            row_num, phpp_glazing_id, phpp_frame_id
        )


def set_baseline_window_area(
    _phpp_conn: phpp_app.PHPPConnection, _baseline_code: BaselineCode
) -> None:
    """Set the maximum window-to-wall ratio in the PHPP Windows Worksheet.

    Arguments:
    ----------
        phpp_conn: phpp_app.PHPPConnection
            The PHPPConnection object
        baseline_code: BaselineCode
            The BaselineCode object
    """
    maximum_wwr = _baseline_code.get_baseline_max_wwr()
    current_wall_area = _phpp_conn.areas.get_total_wall_area()
    current_window_area = _phpp_conn.windows.get_total_window_area()
    current_wwr = current_window_area / (current_wall_area + current_window_area)

    if current_wwr < maximum_wwr:
        print(f"Current WWR {current_wwr:.2%} is less than maximum {maximum_wwr:.0%}.")
        return None

    print(
        f"Current WWR {current_wwr:.2%} is greater than maximum {maximum_wwr:.0%}. Scaling windows."
    )

    # -- Figure out the right scale factor
    scale_factor = maximum_wwr / current_wwr

    # -- Scale the window areas
    for row_num in _phpp_conn.windows.used_window_row_numbers:
        if not _phpp_conn.windows.row_is_window(row_num):
            continue

        _phpp_conn.windows.scale_window_size(row_num, scale_factor)


def set_baseline_skylight_area(
    _phpp_conn: phpp_app.PHPPConnection, _baseline_code: BaselineCode
) -> None:
    """Set the maximum skylight-to-roof ratio in the PHPP Windows Worksheet.

    Arguments:
    ----------
        phpp_conn: phpp_app.PHPPConnection
            The PHPPConnection object
        baseline_code: BaselineCode
            The BaselineCode object
    """
    maximum_srr = _baseline_code.get_baseline_max_srr()
    current_roof_area = _phpp_conn.areas.get_total_roof_area()
    current_skylight_area = _phpp_conn.windows.get_total_skylight_area()
    current_srr = current_skylight_area / (current_roof_area + current_skylight_area)

    if current_srr < maximum_srr:
        print(f"Current SRR {current_srr:.2%} is less than maximum {maximum_srr:.0%}.")
        return None

    print(
        f"Current SRR {current_srr:.2%} is greater than maximum {maximum_srr:.0%}. Scaling Skylights."
    )

    # -- Figure out the right scale factor
    scale_factor = maximum_srr / current_srr

    # -- Scale the window areas
    for row_num in _phpp_conn.windows.used_window_row_numbers:
        if not _phpp_conn.windows.row_is_skylight(row_num):
            continue

        _phpp_conn.windows.scale_window_size(row_num, scale_factor)



# -*- coding: utf-8 -*-
# -*- Python Version: 3.7 -*-

"""Data model of the Baseline Building Code."""

from pydantic import BaseModel, validator
from typing import Dict

from ph_units.converter import convert
from ph_baseliner.codes.options import ClimateZones, Use_Groups, PF_Groups

# -----------------------------------------------------------------------------


class TableMaximumUFactor_Values(BaseModel):
    group_r: float
    all_other: float

    @classmethod
    def convert_units(cls, v: "TableMaximumUFactor_Values", **kwargs):
        for field in v.__fields__:
            val = getattr(v, field)
            val = convert(val, kwargs["units"], "W/m2K")
            setattr(v, field, val)
        return v

    def get_u_values_for_use_group(self, _use_group: Use_Groups) -> float:
        """Return the U-value for the given use group name.

        Args:
            _use_group: Use_Groups:
                The use group to return the U-value for.
        """

        try:
            return getattr(self, _use_group.name)
        except Exception as e:
            allowed_use_group_names = [_ for _ in self.__fields__]
            msg = (
                f"Use Group: '{_use_group.name}' is not supported. "
                f"Supported groups include only: {allowed_use_group_names}"
            )
            raise ValueError(msg, e)


class TableMaximumCFactor_Values(BaseModel):
    group_r: float
    all_other: float

    @classmethod
    def convert_units(cls, v: "TableMaximumCFactor_Values", **kwargs):
        for field in v.__fields__:
            val = getattr(v, field)
            val = convert(val, kwargs["units"], "W/m2K")
            setattr(v, field, val)
        return v

    def get_c_factor_for_use_group(self, _use_group: Use_Groups) -> float:
        """Return the C-Factor for the given use group name.

        Args:
            _use_group: Use_Groups:
                The name of the use group to return the C-Factor for.
        """

        try:
            return getattr(self, _use_group.name)
        except Exception as e:
            allowed_use_group_names = [_ for _ in self.__fields__]
            msg = (
                f"Use Group: '{_use_group.name}' is not supported. "
                f"Supported groups include only: {allowed_use_group_names}"
            )
            raise ValueError(msg, e)


class TableMaximumFFactor_Values(BaseModel):
    group_r: float
    all_other: float

    @classmethod
    def convert_units(cls, v: "TableMaximumFFactor_Values", **kwargs):
        for field in v.__fields__:
            val = getattr(v, field)
            val = convert(val, kwargs["units"], "W/m2K")
            setattr(v, field, val)
        return v

    def get_f_factor_for_use_group(self, _use_group: Use_Groups) -> float:
        """Return the F-Factor for the given use group name.

        Args:
            _use_group: Use_Groups
                The name of the use group to return the F-Factor for.
        """

        try:
            return getattr(self, _use_group.name)
        except Exception as e:
            allowed_use_group_names = [_ for _ in self.__fields__]
            msg = (
                f"Use Group: '{_use_group.name}' is not supported. "
                f"Supported groups include only: {allowed_use_group_names}"
            )
            raise ValueError(msg, e)


class TableMaximumSHGC_Values(BaseModel):
    pf_under_20: float
    pf_20_to_50: float
    pf_over_50: float

    @classmethod
    def convert_units(cls, v: "TableMaximumSHGC_Values", **kwargs):
        return v

    def get_shgc_for_pf(self, _pf_group: PF_Groups) -> float:
        """Return the SHGC value for the given PF group name.

        Args:
            _pf_group: PF_Groups:
                The PF group to return the SHGC for.
        """

        try:
            return getattr(self, _pf_group.name)
        except Exception as e:
            allowed_pf_group_names = [_ for _ in self.__fields__]
            msg = (
                f"Climate Zone: '{_pf_group.name}' is not supported. "
                f"Supported zones include only: {allowed_pf_group_names}"
            )
            raise ValueError(msg, e)


# -----------------------------------------------------------------------------


class TableMaximumUFactor_ClimateZones(BaseModel):
    CZ4: TableMaximumUFactor_Values
    CZ5: TableMaximumUFactor_Values
    CZ6: TableMaximumUFactor_Values

    @classmethod
    def convert_units(cls, v: "TableMaximumUFactor_ClimateZones", **kwargs):
        for field_name in v.__fields__:
            field_type = getattr(v, field_name)
            field_type.convert_units(field_type, **kwargs)
        return v

    def get_u_value_for_climate(
        self, _climate_zone: ClimateZones
    ) -> "TableMaximumUFactor_Values":
        """Return the U-Values for the given climate zone name.

        Args:
            _climate_zone: ClimateZones:
                The name of the climate zone to return the U-Values for.
        """

        try:
            return getattr(self, _climate_zone.name)
        except Exception as e:
            allowed_climate_zone_names = [_ for _ in self.__fields__]
            msg = (
                f"Climate Zone: '{_climate_zone.name}' is not supported. "
                f"Supported zones include only: {allowed_climate_zone_names}"
            )
            raise ValueError(msg, e)


class TableMaximumCFactor_ClimateZones(BaseModel):
    CZ4: TableMaximumCFactor_Values
    CZ5: TableMaximumCFactor_Values
    CZ6: TableMaximumCFactor_Values

    @classmethod
    def convert_units(cls, v: "TableMaximumCFactor_ClimateZones", **kwargs):
        for field_name in v.__fields__:
            field_type = getattr(v, field_name)
            field_type.convert_units(field_type, **kwargs)
        return v

    def get_c_factor_for_climate(
        self, _climate_zone: ClimateZones
    ) -> "TableMaximumCFactor_Values":
        """Return the C-Factors for the given climate zone name.

        Args:
            _climate_zone: ClimateZones
                The name of the climate zone to return the C-Factors for.
        """

        try:
            return getattr(self, _climate_zone.name)
        except Exception as e:
            allowed_climate_zone_names = [_ for _ in self.__fields__]
            msg = (
                f"Climate Zone: '{_climate_zone.name}' is not supported. "
                f"Supported zones include only: {allowed_climate_zone_names}"
            )
            raise ValueError(msg, e)


class TableMaximumFFactor_ClimateZones(BaseModel):
    CZ4: TableMaximumFFactor_Values
    CZ5: TableMaximumFFactor_Values
    CZ6: TableMaximumFFactor_Values

    @classmethod
    def convert_units(cls, v: "TableMaximumFFactor_ClimateZones", **kwargs):
        for field_name in v.__fields__:
            field_type = getattr(v, field_name)
            field_type.convert_units(field_type, **kwargs)
        return v

    def get_f_factor_for_climate(
        self, _climate_zone: ClimateZones
    ) -> "TableMaximumFFactor_Values":
        """Return the F-Factors for the given climate zone name.

        Args:
            _climate_zone: ClimateZones
                The name of the climate zone to return the F-Factors for.
        """

        try:
            return getattr(self, _climate_zone.name)
        except Exception as e:
            allowed_climate_zone_names = [_ for _ in self.__fields__]
            msg = (
                f"Climate Zone: '{_climate_zone.name}' is not supported. "
                f"Supported zones include only: {allowed_climate_zone_names}"
            )
            raise ValueError(msg, e)


class TableMaximumSHGC_ClimateZones(BaseModel):
    CZ4: TableMaximumSHGC_Values
    CZ5: TableMaximumSHGC_Values
    CZ6: TableMaximumSHGC_Values

    @classmethod
    def convert_units(cls, v: "TableMaximumSHGC_ClimateZones", **kwargs):
        return v

    def get_shgcs_for_climate(
        self, _climate_zone: ClimateZones
    ) -> TableMaximumSHGC_Values:
        """Get the SHGC values for a given climate zone.

        Args:
            climate_zone: ClimateZones:
                The name of the climate zone.
        """
        try:
            return getattr(self, _climate_zone.name)
        except Exception as e:
            allowed_climate_zone_names = [_ for _ in self.__fields__]
            msg = (
                f"Climate Zone: '{_climate_zone.name}' is not supported. "
                f"Supported zones include only: {allowed_climate_zone_names}"
            )
            raise ValueError(msg, e)


# -----------------------------------------------------------------------------


class TableMaximumUFactorRoofs(BaseModel):
    insulation_above_deck: TableMaximumUFactor_ClimateZones
    metal_building: TableMaximumUFactor_ClimateZones
    attic_and_other: TableMaximumUFactor_ClimateZones

    @classmethod
    def convert_units(cls, v: "TableMaximumUFactorRoofs", **kwargs):
        for field_name in v.__fields__:
            field_type = getattr(v, field_name)
            field_type.convert_units(field_type, **kwargs)
        return v


class TableMaximumUFactorWalls(BaseModel):
    mass: TableMaximumUFactor_ClimateZones
    metal_building: TableMaximumUFactor_ClimateZones
    metal_framed: TableMaximumUFactor_ClimateZones
    wood_framed_and_other: TableMaximumUFactor_ClimateZones
    below_grade: TableMaximumCFactor_ClimateZones

    @classmethod
    def convert_units(cls, v: "TableMaximumUFactorWalls", **kwargs):
        for field_name in v.__fields__:
            field_type = getattr(v, field_name)
            field_type.convert_units(field_type, **kwargs)
        return v


class TableMaximumUFactorFloors(BaseModel):
    mass: TableMaximumUFactor_ClimateZones
    joist_framed: TableMaximumUFactor_ClimateZones
    unheated_slab: TableMaximumFFactor_ClimateZones
    heated_slab: TableMaximumUFactor_ClimateZones

    @classmethod
    def convert_units(cls, v: "TableMaximumUFactorFloors", **kwargs):
        for field_name in v.__fields__:
            field_type = getattr(v, field_name)
            field_type.convert_units(field_type, **kwargs)
        return v


class TableMaximumUFactorFenestration(BaseModel):
    u_factors: TableMaximumUFactor_ClimateZones
    shgc: TableMaximumSHGC_ClimateZones

    @classmethod
    def convert_units(cls, v: "TableMaximumUFactorFenestration", **kwargs):
        for field_name in v.__fields__:
            field_type = getattr(v, field_name)
            field_type.convert_units(field_type, **kwargs)
        return v


# -----------------------------------------------------------------------------


class TableEnvelopeMaximumUFactors(BaseModel):
    name: str
    section: str
    units: str
    roofs: TableMaximumUFactorRoofs
    walls: TableMaximumUFactorWalls
    floors: TableMaximumUFactorFloors

    @validator("roofs", "walls", "floors", pre=False)
    @classmethod
    def convert_units(cls, v: "TableEnvelopeMaximumUFactors", values: Dict):
        return type(v).convert_units(v, **values)


class TableFenestrationMaximumUFactors(BaseModel):
    name: str
    section: str
    units: str
    fixed_windows: TableMaximumUFactorFenestration
    operable_windows: TableMaximumUFactorFenestration
    entrance_doors: TableMaximumUFactorFenestration
    skylights: TableMaximumUFactorFenestration

    @validator(
        "fixed_windows", "operable_windows", "entrance_doors", "skylights", pre=False
    )
    @classmethod
    def convert_units(cls, v: "TableFenestrationMaximumUFactors", values: Dict):
        return type(v).convert_units(v, **values)


class TableLightingLPDAreaMethod(BaseModel):
    name: str
    section: str
    units: str
    LPD: dict

    @validator("LPD", pre=False)
    @classmethod
    def convert_units(cls, v: Dict, values: Dict):
        units = values["units"]
        for key, val in v.items():
            v[key] = convert(val, units, "W/m2")
        return v


# -----------------------------------------------------------------------------


class SectionMaximumFenestrationArea(BaseModel):
    source_url: str
    name: str
    section: str
    maximum_skylight_percent_of_roof: float
    maximum_window_percent_of_wall: float


# -----------------------------------------------------------------------------


class BaselineCodeTables(BaseModel):
    maximum_u_factors: TableEnvelopeMaximumUFactors
    fenestration_u_factors: TableFenestrationMaximumUFactors
    lighting_area_method: TableLightingLPDAreaMethod


class BaselineCodeSections(BaseModel):
    maximum_fenestration_area: SectionMaximumFenestrationArea


class BaselineCode(BaseModel):
    """Pydantic BaselineCode model."""

    name: str
    state: str
    year: str
    source_url: str
    tables: BaselineCodeTables
    sections: BaselineCodeSections

    def get_window_shgcs(self) -> TableMaximumSHGC_ClimateZones:
        """Return the TableMaximumSHGC_ClimateZones with the window SHGC values.

        Returns:
            * TableMaximumSHGC_ClimateZones: The SHGC values for windows.
        """
        return self.tables.fenestration_u_factors.operable_windows.shgc

    def get_window_u_values(self) -> TableMaximumUFactor_ClimateZones:
        """Return the TableMaximumUFactor_ClimateZones with the window U-values.

        Returns:
            * TableMaximumUFactor_ClimateZones: The U-values for windows.
        """
        return self.tables.fenestration_u_factors.operable_windows.u_factors

    def get_roof_u_values(self) -> TableMaximumUFactor_ClimateZones:
        """Get the roof groups from the baseline code."""
        return self.tables.maximum_u_factors.roofs.insulation_above_deck

    def get_exposed_wall_u_values(self) -> TableMaximumUFactor_ClimateZones:
        """Get the exposed wall groups from the baseline code."""
        return self.tables.maximum_u_factors.walls.metal_framed

    def get_ground_wall_c_factors(self) -> TableMaximumCFactor_ClimateZones:
        """Get the ground wall groups from the baseline code."""
        return self.tables.maximum_u_factors.walls.below_grade

    def get_exposed_floor_u_values(self) -> TableMaximumUFactor_ClimateZones:
        """Get the exposed floor groups from the baseline code."""
        return self.tables.maximum_u_factors.floors.mass

    def get_ground_floor_f_factors(self) -> TableMaximumFFactor_ClimateZones:
        """Get the ground floor groups from the baseline code."""
        return self.tables.maximum_u_factors.floors.unheated_slab

    def get_baseline_max_wwr(self) -> float:
        """Get the maximum window-to-wall ratio from the baseline code."""
        return self.sections.maximum_fenestration_area.maximum_window_percent_of_wall

    def get_baseline_max_srr(self) -> float:
        """Get the maximum skylight-to-roof ratio from the baseline code."""
        return self.sections.maximum_fenestration_area.maximum_skylight_percent_of_roof

    def get_baseline_roof_u_value(self, _cz: ClimateZones, _ug: Use_Groups) -> float:
        """Get the maximum roof U-value from the baseline code for a climate/use-group."""
        all_roof_u_values = self.get_roof_u_values()
        cz_roof_u_values = all_roof_u_values.get_u_value_for_climate(_cz)
        roof_u_value = cz_roof_u_values.get_u_values_for_use_group(_ug)
        return roof_u_value

    def get_baseline_exposed_wall_u_value(
        self, _cz: ClimateZones, _ug: Use_Groups
    ) -> float:
        """Get the maximum exposed wall U-value from the baseline code for a climate/use-group."""
        all_exposed_wall_u_values = self.get_exposed_wall_u_values()
        cz_exposed_wall_u_values = all_exposed_wall_u_values.get_u_value_for_climate(
            _cz
        )
        exposed_wall_u_value = cz_exposed_wall_u_values.get_u_values_for_use_group(_ug)
        return exposed_wall_u_value

    def get_baseline_ground_wall_c_factor(
        self, _cz: ClimateZones, _ug: Use_Groups
    ) -> float:
        """Get the maximum ground wall C-Factor from the baseline code for a climate/use-group."""
        all_ground_wall_c_factors = self.get_ground_wall_c_factors()
        cz_ground_wall_c_factors = all_ground_wall_c_factors.get_c_factor_for_climate(
            _cz
        )
        ground_wall_c_factor = cz_ground_wall_c_factors.get_c_factor_for_use_group(_ug)
        return ground_wall_c_factor

    def get_baseline_exposed_floor_u_value(
        self, _cz: ClimateZones, _ug: Use_Groups
    ) -> float:
        """Get the maximum exposed floor U-value from the baseline code for a climate/use-group."""
        all_exposed_floor_u_values = self.get_exposed_floor_u_values()
        cz_exposed_floor_u_values = all_exposed_floor_u_values.get_u_value_for_climate(
            _cz
        )
        exposed_floor_u_value = cz_exposed_floor_u_values.get_u_values_for_use_group(
            _ug
        )
        return exposed_floor_u_value

    def get_baseline_ground_floor_f_factor(
        self, _cz: ClimateZones, _ug: Use_Groups
    ) -> float:
        """Get the maximum ground floor F-Factor from the baseline code for a climate/use-group."""
        all_ground_floor_f_factors = self.get_ground_floor_f_factors()
        cz_ground_floor_f_factors = all_ground_floor_f_factors.get_f_factor_for_climate(
            _cz
        )
        ground_floor_f_factor = cz_ground_floor_f_factors.get_f_factor_for_use_group(
            _ug
        )
        return ground_floor_f_factor



# -*- coding: utf-8 -*-
# -*- Python Version: 3.7 -*-

"""Allowable options for the Baseline Codes."""

from enum import Enum
from typing import List


class ClimateZones(Enum):
    """The climate zones that are available for the PH Baseliner."""

    CZ4 = "CZ4 (Except Marine 4)"
    CZ5 = "CZ5 (and Marine 4)"
    CZ6 = "CZ6"

    @classmethod
    def as_list(cls) -> List[str]:
        """Return a list of the Enum values as strings."""
        return [_.value for _ in cls]


class BaselineCodes(Enum):
    """The baseline codes that are available for the PH Baseliner."""

    ECCCNYS_2020 = "ECC of NYS 2020"

    @classmethod
    def as_list(cls) -> List[str]:
        """Return a list of the Enum values as strings."""
        return [_.value for _ in cls]


class PF_Groups(Enum):
    """The Projection Factor groups that are available for the PH Baseliner."""

    pf_under_20 = "Under 0.2"
    pf_20_to_50 = "0.2 to 0.5"
    pf_over_50 = "Over 0.5"

    @classmethod
    def as_list(cls) -> List[str]:
        """Return a list of the Enum values as strings."""
        return [_.value for _ in cls]


class Use_Groups(Enum):
    """The Use groups that are available for the PH Baseliner."""

    group_r = "Group R"
    all_other = "All Other"

    @classmethod
    def as_list(cls) -> List[str]:
        """Return a list of the Enum values as strings."""
        return [_.value for _ in cls]



# -*- coding: utf-8 -*-
# -*- Python Version: 3.7 -*-

"""Equivalence map between PHPP-space-type-names and Code-space-type-names."""

# -- TABLE C405.3.2(1) INTERIOR LIGHTING POWER ALLOWANCES: BUILDING AREA METHOD
# -- PHPP-Space-Type-Name : Code-Space-Type-Name
space_type_map = {
    "Single office": "Office",
    "Group office": "Office",
    "Open-plan office": "Office",
    "Meeting": "Office",
    "Counter area": "Retail",
    "Retail": "Retail",
    "Retail/department store with refrigerated products": "Retail",
    "Classroom": "School/University",
    "University auditorium": "School/University",
    "Bedroom": "Dormitory",
    "Hotel room": "Hotel/Motel",
    "Canteen": "Dining: cafeteria/fast food",
    "Restaurant": "Dining: family",
    "Kitchen in non-residential building": "Dining: family",
    "Kitchen, Storage, Preparation": "Dining: family",
    "WC, Sanitary": "Office",
    "Other habitable rooms": "Office",
    "Secondary areas": "Office",
    "Circulation area": "Office",
    "Storage, Services": "Office",
    "Server room": "Office",
    "Workshop (standing activity)": "Workshop",
    "Workshop (predominantly standing activity)": "Workshop",
    "Workshop (predominantly sedentary activity)": "Workshop",
    "Theatre auditorium": "Performing arts theater",
    "Theatre foyer": "Performing arts theater",
    "Theatre stage": "Performing arts theater",
    "Fair, Congress": "Courthouse",
    "Exhibition": "Museum",
    "Library reading room": "Library",
    "Open access library": "Library",
    "Library repository":"Library",
    "Sports hall": "Sports arena",
    "Parking garage": "Parking garage",
    "Public parking garage": "Parking garage",
    "Sauna area": "Exercise center",
    "Exercise room": "Exercise center",
    "Laboratory": "Health care clinic",
    "Examination/treatment rooms": "Health care clinic",
    "Special care areas": "Health care clinic",
    "Corridors of the general care area": "Health care clinic",
    "Medical/therapeutic practices": "Health care clinic",
    "Warehouses": "Warehouse",
}
